/* ------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG 2009-2014

   Project:  Eloquence JDLG Plugin Interface
   File:     Line.java
   Function: JDLG Charts Plugin: Line Chart
   $Id: Line.java,v 1.7 2014/12/11 12:17:23 rhg Exp $

   Edit history:
   2009-04-06, ap   - creation
   2009-04-28, mike - finalized
   2014-04-08, rhg  - api version 1.1 additions
   2014-12-03, rhg  - revised, enhancements, bugfixes, code cleanup
 ------------------------------------------------------------------------ */

package com.eloquence.plugin.charts;

import com.eloquence.api.*;

import java.awt.*;
import java.util.*;

import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.category.*;
import org.jfree.chart.labels.*;
import org.jfree.ui.*;

public class Line
   extends Axis
{
   // ------------------------------------------------------------------------
   // Plugin
   // ------------------------------------------------------------------------

   public Component onCreate(Jdlg jdlg)
   {
      if (!jdlg.requiresApiVersion(1,0))
         return null;

      // label: different default value
      showLabels = false;

      peerPanel = new ChartPanel(null);
      return peerPanel;
   }

   public void onEloq2Swing()
   {
      if (chartModified)
      {
         // createLineChart(java.lang.String title,
         //               java.lang.String categoryAxisLabel,
         //               java.lang.String valueAxisLabel,
         //               CategoryDataset dataset,
         //               PlotOrientation orientation,
         //               boolean legend,
         //               boolean tooltips,
         //               boolean urls)

         String titleLabel = title.length() > 0 ? title : null;
         final String categoryLabel = "Category";
         final String valueLabel = "Value";

         if (paint3d)
            peerPanel.setChart(
               ChartFactory.createLineChart3D(
                  titleLabel, categoryLabel, valueLabel,
                  dataset, PlotOrientation.VERTICAL,
                  legend, true, false));
         else
            peerPanel.setChart(
               ChartFactory.createLineChart(
                  titleLabel, categoryLabel, valueLabel,
                  dataset, PlotOrientation.VERTICAL,
                  legend, true, false));

         chartModified = false;
         attributeModified = true;
         datasetModified = true;
      }

      super.onEloq2Swing();

      JFreeChart chart = null;
      CategoryPlot plot = null;
      LineAndShapeRenderer renderer = null;

      if (datasetModified)
      {
         if (chart == null)
         {
            chart = peerPanel.getChart();
            plot = (CategoryPlot)chart.getPlot();
            renderer = (LineAndShapeRenderer)plot.getRenderer();
         }

         int numRows = rows.size();
         for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
         {
            Row row = (Row)rows.get(rowIndex);
            renderer.setSeriesPaint(rowIndex, row.getColor());
         }
      }

      if (attributeModified)
      {
         if (chart == null)
         {
            chart = peerPanel.getChart();
            plot = (CategoryPlot)chart.getPlot();
            renderer = (LineAndShapeRenderer)plot.getRenderer();
         }

         chart.setBorderVisible(border);
         chart.setPadding(borderInsets);
         plot.setBackgroundPaint(chartBgc);
         plot.setForegroundAlpha((float)fgAlpha / 100);
      }

      if (datasetModified || attributeModified)
      {
         boolean shapesVisible = dots != 0;
         boolean shapesFilled = dots > 1;

         renderer.setDrawOutlines(shapesVisible);
         renderer.setBaseShapesVisible(shapesVisible);
         renderer.setBaseShapesFilled(shapesFilled);

         StandardCategoryItemLabelGenerator labelGenerator;
         if (showLabels)
            labelGenerator = new StandardCategoryItemLabelGenerator(
               getLabelFormat(), valueFormat(), percentFormat());
         else
            labelGenerator = null;

         int numRows = rows.size();
         for (int rowIndex = 0; rowIndex < numRows; rowIndex++)
         {
            renderer.setSeriesShapesVisible(rowIndex, shapesVisible);
            renderer.setSeriesShapesFilled(rowIndex, shapesFilled);

            renderer.setSeriesItemLabelGenerator(rowIndex, labelGenerator);
            renderer.setSeriesItemLabelsVisible(rowIndex, showLabels);
         }
      }

      datasetModified = false;
      attributeModified = false;
   }

   // ------------------------------------------------------------------------
   // get/set paint3d
   // ------------------------------------------------------------------------

   private static final String PAINT3D_ATTR_NAME = "paint3d";
   private boolean paint3d = false;

   private final void setPaint3d(int paint3d)
   {
      this.paint3d = paint3d != 0;
      chartModified = true;
   }

   private final int getPaint3d()
   {
      return paint3d ? 1 : 0;
   }

   // ------------------------------------------------------------------------
   // get/set dots
   // ------------------------------------------------------------------------

   private static final String DOTS_ATTR_NAME = "dots";
   private int dots = 0;

   private final void setDots(int dots)
   {
      this.dots = dots;
      chartModified = true;
   }

   private final int getDots()
   {
      return dots;
   }

   // ------------------------------------------------------------------------
   // attributes
   // ------------------------------------------------------------------------

   private static final int PAINT3D_ATTR_ID = 1;
   private static final int DOTS_ATTR_ID = 2;

   public void onClone(Plugin clone)
   {
      super.onClone(clone);

      Line cloneObj = (Line)clone;
      cloneObj.setPaint3d(getPaint3d());
      cloneObj.setDots(getDots());
   }

   public boolean onSet(String attribute, java.util.List index,
                        UnifiedData data)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case PAINT3D_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setPaint3d(data.getIntData());
               break;

            case DOTS_ATTR_ID:
               if (indexElements != 0)
                  return false;
               if (data.isStringData())
                  return false;
               setDots(data.getIntData());
               break;
         }

         return true;
      }

      return super.onSet(attribute, index, data);
   }

   public UnifiedData onGet(String attribute, java.util.List index)
   {
      Integer attrInt = (Integer)attributeList.get(attribute);
      if (attrInt != null)
      {
         int indexElements = index != null ? index.size() : 0;
         switch (attrInt.intValue())
         {
            case PAINT3D_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getPaint3d());

            case DOTS_ATTR_ID:
               if (indexElements != 0)
                  return null;
               return new UnifiedData(getDots());
         }
      }

      return super.onGet(attribute, index);
   }

   private static HashMap attributeList = new HashMap();
   static
   {
      attributeList.put(PAINT3D_ATTR_NAME, new Integer(PAINT3D_ATTR_ID));
      attributeList.put(DOTS_ATTR_NAME, new Integer(DOTS_ATTR_ID));
   }
}
